<?php
// Copyright 1999-2023. Plesk International GmbH. All rights reserved.

declare(strict_types=1);

include_once(__DIR__ . DIRECTORY_SEPARATOR . '../../../vendor' . DIRECTORY_SEPARATOR . 'autoload.php');

require '../../../init.php';

use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Hosting;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Product;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\ProductCustomField;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Server;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\ServerGroup;
use WHMCS\Module\Server\SolusVm2Vps\Helpers\Arr;
use WHMCS\Module\Server\SolusVm2Vps\Logger\Logger;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Connector;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\ComputeResourceResource;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\LocationResource;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\OsImageResource;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\PlanResource;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\RoleResource;

class Converter
{
    private const SolusVMv1 = 'solusvmpro';
    private const SolusVMv2 = 'solusvm2vps';

    // keys
    private const Server = '--mn-server';
    private const PackageV1 = '--v1-package';
    private const PackageV2 = '--v2-package';
    private const KVMPlan = '--kvm-plan';
    private const VZPlan = '--vz-plan';
    private const VZOsImage = '--vz-os-image';
    private const KVMOsImage = '--kvm-os-image';
    private const Location = '--location';
    private const Role = '--role';
    private const LimitGroup = '--limit-group';

    private array $arguments;

    public function __construct(array $argv)
    {
        $this->arguments = $argv;
    }

    public function start(): int
    {
        $command = 'help';
        if (isset($this->arguments[1])) {
            $command = $this->arguments[1];
        }

        switch ($command) {
            case 'products':
                $this->products();
                break;
            case 'run':
                return $this->run();
            case 'reconfigure':
                return $this->reconfigure();
            case 'help':
            default:
                $this->help();
        }
        return 0;
    }

    private function help(): void
    {
        echo <<<EOT
SolusVM2 Converter
This command allows you to convert VPS`es of SolusVM v1 Products to VPS`es of SolusVM v2 Products
Available commands:
* help - shows available commands
* products - shows all existing SolusVM v1 and SolusVM v2 products
* run {ProductSolusVMv1ID} {ProductSolusVMv2ID} - converts all VPS`es of SolusVM v1 to SolusVM v2. Where ProductSolusVMv1ID - ID of SolusVM v1 product which you want to convert to SolusVM v2, ProductSolusVMv2ID - ID of SolusVM v2 product
* reconfigure - reconfigures all SolusVM v1 products into the SolusVM v2 products. To be sure that all products will be converted run this command with all available flags. Available flags:
    --mn-server - ID of SolusVM v2 server (required)
    --kvm-plan - ID of SolusVM v2 plan with virtualization type KVM, needed if we can't find plan by plan name which is used in SolusVM v1 product
    --vz-plan - ID of SolusVM v2 plan with virtualization type VZ, needed if we can't find plan by plan name which is used in SolusVM v1 product
    --kvm-os-image - ID of SolusVM v2 OS image with virtualization type KVM, needed for KVM products because we don't import KVM OS images from SolusVM v1
    --vz-os-image - ID of SolusVM v2 OS image with virtualization type VZ, needed if we can't find OS image by template which is used in SolusVM v1 product
    --location - ID of SolusVM v2 location, needed if we can't find location by Node Group name which is used in SolusVM v1 product
    --role - ID of SolusVM v2 role (optional, default value is ID of role with name "CLIENT")
    --limit-group - ID of SolusVM v2 limit group (optional)
EOT;
    }

    private function products(): void
    {
        $products = Product::getByType(self::SolusVMv1);
        if ($products) {
            echo "Existing SolusVM v1 Products:\nID \t Name\n";
            foreach ($products as $product) {
                echo "$product->id \t $product->name\n";
            }
        }

        $products = Product::getByType(self::SolusVMv2);
        if ($products) {
            echo "Available for converting SolusVM v2 Products:\nID \t Name\n";
            foreach ($products as $product) {
                echo "$product->id \t $product->name\n";
            }
        }
    }

    private function run(): int
    {
        if (!isset($this->arguments[2]) || !isset($this->arguments[3])) {
            echo "Wrong number of arguments. Use `help` command to see how to use this command.";
            return 1;
        }

        $v1ID = (int)$this->arguments[2];
        $v1Product = Product::getById($v1ID);
        if (!$v1Product) {
            echo "Can't find SolusVM v1 product by ID: $v1ID";
            return 1;
        }
        if ($v1Product->servertype !== self::SolusVMv1) {
            echo "Server type Product ID $v1ID is $v1Product->servertype, but expected server type is ".self::SolusVMv1;
            return 1;
        }

        $v2ID = (int)$this->arguments[3];
        $v2Product = Product::getById($v2ID);
        if (!$v2Product) {
            echo "Can't find SolusVM v2 product by ID: $v2ID";
            return 1;
        }
        if ($v2Product->servertype !== self::SolusVMv2) {
            echo "Server type Product ID $v2ID is $v2Product->servertype, but expected server type is ".self::SolusVMv2;
            return 1;
        }

        $serverGroup = ServerGroup::getById($v2Product->servergroup);
        if (!$serverGroup) {
            echo "Failed to find Server Group of passed SolusVM v2 Product. ProductID = $v2ID; ServerGroupID = $v2Product->servergroup;";
            return 1;
        }

        $servers = Hosting::getByProductID($v1ID);
        if (!$servers) {
            echo "SolusVM v1 Product ID $v1ID doesn't have any servers to convert.";
            return 1;
        }

        echo "Founded servers:\nID \t Domain\n";
        foreach ($servers as $server) {
            echo "$server->id \t $server->domain\n";
        }

        echo "Do you want to continue? (y/N)\n";

        $confirmation = readline();
        if (strtolower(trim($confirmation)) !== 'y') {
            return 0;
        }

        Hosting::updateByPackage($v1ID, [
            'packageid' => $v2ID,
            'server' => $serverGroup->serverid,
        ]);
        echo 'Done!';
        return 0;
    }

    public function reconfigure(): int
    {
        $serverID = $this->getArgument(self::Server);
        if (!$serverID) {
            echo "Server ID is required. Use `help` command to see how to use this command.";
            return 1;
        }
        $serverParams = Server::getParams($serverID);
        $connector = Connector::create($serverParams);

        $serverGroup = ServerGroup::getByServerId($serverID);
        if (!$serverGroup) {
            echo "Server group is not found";
            return 1;
        }

        // arguments
        $kvmPlan = $this->getArgument(self::KVMPlan);
        $vzPlan = $this->getArgument(self::VZPlan);
        $kvmOsImage = $this->getArgument(self::KVMOsImage);
        $vzOsImage = $this->getArgument(self::VZOsImage);
        $location = $this->getArgument(self::Location);
        $role = $this->getArgument(self::Role);
        $limitGroup = $this->getArgument(self::LimitGroup);

        if (!$role) {
            $roleResource = new RoleResource($connector);
            $roles = $roleResource->list();
            foreach ($roles as $item) {
                if (Arr::get($item, 'name') === 'CLIENT') {
                    $role = Arr::get($item, 'id');
                    break;
                }
            }
        }

        Logger::log([
            self::KVMPlan => $kvmPlan,
            self::VZPlan => $vzPlan,
            self::KVMOsImage => $kvmOsImage,
            self::VZOsImage => $vzOsImage,
            self::Location => $location,
            self::Role => $role,
            self::LimitGroup => $limitGroup,
        ], 'Start reconfiguring products');

        // lists
        $planResource = new PlanResource($connector);
        $plans = $planResource->list();
        $osImageResource = new OsImageResource($connector);
        $osImages = $osImageResource->list();
        $locationResource = new LocationResource($connector);
        $locations = $locationResource->list();
        $crResource = new ComputeResourceResource($connector);
        $computeResources = $crResource->list();

        $notSupportedProducts = [];
        $convertedProducts = [];

        $products = Product::getByType(self::SolusVMv1);
        foreach ($products as $product) {

            $productCR = $product->configoption2;
            $productPlan = $product->configoption4;
            $productLocation = $product->configoption9;
            $productVirtualizationType = $product->configoption5;
            $productOsImage = $product->configoption6;

            if ($productVirtualizationType !== 'OpenVZ' && $productVirtualizationType !== 'KVM') {
                $notSupportedProducts[] = [
                    "$product->id | $product->name" => "Not supported virtualization type $productVirtualizationType",
                ];
                continue;
            }

            $virtualizationType = $productVirtualizationType === 'OpenVZ' ? 'vz' : 'kvm';

            // get Plan
            $planId = 0;
            foreach ($plans as $item) {
                if (Arr::get($item, 'name') === $productPlan) {
                    $planId = Arr::get($item, 'id');
                }
            }
            if (!$planId) {
                $planId = $virtualizationType === 'vz' ? $vzPlan : $kvmPlan;
                if (!$planId) {
                    $notSupportedProducts[] = [
                        "$product->id | $product->name" => "Not found plan for virtualization type `$virtualizationType`",
                    ];
                    continue;
                }
            }

            // get OS Image
            $osImageID = 0;
            if ($virtualizationType === 'vz') { // We can find only VZ OS Image
                foreach ($osImages as $item) {
                    foreach ($item['versions'] as $version) {
                        if (Arr::get($version, 'url') === $productOsImage) {
                            $osImageID = Arr::get($version, 'id');
                            break;
                        }
                    }
                }
            }
            if (!$osImageID) {
                $osImageID = $virtualizationType === 'vz' ? $vzOsImage : $kvmOsImage;
                if (!$osImageID) {
                    $notSupportedProducts[] = [
                        "$product->id | $product->name" => "Not found OS image for virtualization type `$virtualizationType`",
                    ];
                    continue;
                }
            }

            // get Location
            $locationID = 0;
            if ($productLocation) {
                foreach ($locations as $item) {
                    if (Arr::get($item, 'name') === $productLocation) {
                        $locationID = Arr::get($item, 'id');
                        break;
                    }
                }
            } else if ($productCR) {
                foreach ($computeResources as $computeResource) {
                    if (Arr::get($computeResource, 'name') === $productCR) {
                        foreach (Arr::get($computeResource, 'locations') as $item) {
                            $locationID = Arr::get($item, 'id');
                            break;
                        }
                    }
                }
            }
            if (!$locationID) {
                $locationID = $location;
                if (!$locationID) {
                    $notSupportedProducts[] = [
                        "$product->id | $product->name" => "Not found location for virtualization type `$virtualizationType`",
                    ];
                    continue;
                }
            }

            $config = [
                'servertype' => self::SolusVMv2,
                'servergroup' => $serverGroup->groupid,
                'configoption1' => $planId,
                'configoption2' => $locationID,
                'configoption3' => $osImageID,
                'configoption4' => '',
                'configoption5' => '',
                'configoption6' => '',
                'configoption7' => $role,
                'configoption8' => $limitGroup > 0 ? $limitGroup : '',
                'configoption9' => '',
                'configoption10' => '',
                'configoption11' => '',
                'configoption12' => '',
                'configoption13' => '',
                'configoption14' => '',
                'configoption15' => '',
                'configoption16' => '',
                'configoption17' => '',
                'configoption18' => '',
                'configoption19' => '',
                'configoption20' => '',
                'configoption21' => '',
                'configoption22' => '',
                'configoption23' => '',
                'configoption24' => '',
            ];

            Product::update($product->id, $config);
            ProductCustomField::deleteByProductId($product->id);
            Hosting::updateByPackage($product->id, ['server' => $serverID]);

            $convertedProducts[$product->id] = $product->name;
        }

        if (count($notSupportedProducts)) {
            Logger::log($notSupportedProducts, "Not supported products during reconfiguring");
        }

        if (count($convertedProducts)) {
            Logger::log($convertedProducts, "Converted products");
        }

        echo 'Done! Number of converted products: '.count($convertedProducts);
        return 0;
    }

    private function getArgument(string $key): int
    {
        foreach ($this->arguments as $i => $item) {
            $argument = strtolower($item);

            if ($argument === $key && isset($this->arguments[$i + 1]) && is_numeric($this->arguments[$i + 1])) {
                return (int)$this->arguments[$i + 1];
            }

            $s = explode('=', $argument);
            if (count($s) === 2 && trim($s[0]) === $key && is_numeric($s[1])) {
                return (int)$s[1];
            }
        }

        return 0;
    }
}

$c = new Converter($argv);
$c->start();
echo "\n";